@@ -30,9 +30,11 @@
 #include <sys/types.h>
 #include <dirent.h>
 #include <signal.h>
-#include <gnome.h>
+#include <unistd.h>
+#include <errno.h>
 #include <glib/gstdio.h>
 #include <glib/gi18n.h>
+#include <dbus/dbus-glib.h>
 #include <cspi/spi.h>
 #include <libbonobo.h>
 #include <bonobo-activation/bonobo-activation.h>
@@ -102,6 +104,13 @@ static Display *xevie_dpy = NULL;
 #define GCONF_ACCESSIBILITY_KEY "/desktop/gnome/interface/accessibility"
 #define KEYBOARD_ACCESSIBILITY_ENABLE_KEY "/desktop/gnome/accessibility/keyboard/enable"
 
+/* GNOME Session Manager */
+#define GSM_SERVICE_DBUS       "org.gnome.SessionManager"
+#define GSM_PATH_DBUS          "/org/gnome/SessionManager"
+#define GSM_INTERFACE_DBUS     "org.gnome.SessionManager"
+
+#define GSM_LOGOUT_MODE_NORMAL 0
+
 static gboolean respawn_on_segv = TRUE;
 
 static guint input_idle_handler = 0;
@@ -431,7 +440,7 @@ gok_args_init (GokArgs *args)
 	args->custom_compose_kbd_name = NULL;
 	args->display_keyboard_editor = FALSE;
 	args->geometry = NULL;
-	args->geometry_bitmask;
+	args->geometry_bitmask = 0;
 	args->is_login = 0;
 	args->debug_nameless = FALSE;
 	args->remember_geometry = FALSE;
@@ -641,9 +650,9 @@ gok_main_center_corepointer (GtkWidget *window)
 gint
 gok_main_open(gint argc, gchar *argv[])
 {
-	GnomeProgram *gok_program;
 	GOptionContext *context;
-	gboolean create_dirs;
+	GError *error = NULL;
+	gboolean create_dirs, parsed;
 	int i;
 
 	/* initialize member data */
@@ -656,23 +665,22 @@ gok_main_open(gint argc, gchar *argv[])
 
 	gok_args_init (&gok_args);
 
-	context	= g_option_context_new (_("The GNOME On-screen Keyboard"));
+	context = g_option_context_new (_("The GNOME On-screen Keyboard"));
 	g_option_context_add_main_entries (context, options, GETTEXT_PACKAGE);
+	g_option_context_add_group (context, gtk_get_option_group (TRUE));
+	parsed = g_option_context_parse (context, &argc, &argv, &error);
+	g_option_context_free (context);
+	if (!parsed)
+	{
+		g_print ("%s\n", error->message);
+		g_error_free (error);
+		_exit (-1);
+	}
 
 	/* If the user does not have a writable HOME directory, then
 	   avoid creating the directory. */
 	create_dirs = (g_access (g_get_home_dir(), W_OK) == 0);
 
-	/* Initialize Gnome */
-	gok_program = gnome_program_init (PACKAGE, VERSION,
-		LIBGNOMEUI_MODULE, argc, argv,
-		GNOME_PARAM_GOPTION_CONTEXT, context,
-		GNOME_PROGRAM_STANDARD_PROPERTIES,
-	        GNOME_PARAM_APP_DATADIR, DATADIR"/gok", 
-		LIBGNOMEUI_PARAM_CRASH_DIALOG, FALSE,
-		GNOME_PARAM_CREATE_DIRECTORIES, create_dirs,
-		NULL);
-		
 	if (!bonobo_init (&argc, argv))
 	{
 	    if (gok_args.is_login)
@@ -1010,8 +1018,6 @@ gok_log ("finished check");
 	signal(SIGTERM, (void (*)(int))gok_sig_handler);
 	signal(SIGINT,  (void (*)(int))gok_sig_handler);
 
-	g_object_unref (gok_program);
-
 	return 0;
 }
 
@@ -3166,24 +3172,72 @@ static GtkWidget* _corepointer_warning = NULL; /* core pointer warning dialog */
 
 static gboolean _corepointer_warning_posted = FALSE; /* core pointer warning dialog has been posted */
 
-void check_accessibility_cb ( GObject* o, gpointer* data )
+/**
+ * gok_main_request_logout:
+ *
+ * Calls the 'Logout' method of gnome-session-manager over DBus.
+ * The 'mode' argument is set to 0 (normal).
+ * Other modes are 1 (no confirmation) and 2 (force).
+ *
+ * Returns: #TRUE on success, #FALSE otherwise.
+ */
+static gboolean
+gok_main_request_logout (void)
 {
-	GnomeClient *client;
+	DBusGConnection *bus;
+	DBusGProxy *sm_proxy;
+	GError *error = NULL;
+	gboolean ret = FALSE;
 
-	if (acd) { 
-	    gtk_widget_destroy(acd); acd = NULL; 
-	}	
-	
+	gok_log_enter ();
+
+	/* get session bus */
+	bus = dbus_g_bus_get (DBUS_BUS_SESSION, NULL);
+	if (bus)
+	{
+		/* create g-s-m proxy */
+		sm_proxy = dbus_g_proxy_new_for_name (bus,
+						      GSM_SERVICE_DBUS,
+						      GSM_PATH_DBUS,
+						      GSM_INTERFACE_DBUS);
+		/* request logout (sync. call) */
+		ret = dbus_g_proxy_call (sm_proxy, "Logout", &error,
+					 G_TYPE_UINT, GSM_LOGOUT_MODE_NORMAL,
+					 G_TYPE_INVALID, G_TYPE_INVALID);
+		g_object_unref (sm_proxy);
+		if (error)
+		{
+			gok_log ("Logout method: %s", error->message);
+			g_error_free (error);
+		}
+	}
+	else
+	{
+		gok_log ("Couldn't connect to session bus.");
+	}
+	gok_log_leave ();
+
+	return ret;
+}
+
+static void
+check_accessibility_cb (GtkButton *button, gpointer data)
+{
+	if (acd) {
+		gtk_widget_destroy(acd);
+		acd = NULL;
+	}
+
 	if (strcmp((gchar*)data,"logout") == 0) {
-		gok_gconf_set_bool ( gok_data_get_gconf_client(),
-			GCONF_ACCESSIBILITY_KEY, TRUE );
-		if (!(client = gnome_master_client())) {
-			gok_main_close();
+		/* enable gnome a11y key */
+		gok_gconf_set_bool (gok_data_get_gconf_client (),
+				    GCONF_ACCESSIBILITY_KEY, TRUE);
+
+		/* request logout */
+		if (!gok_main_request_logout ()) {
+			/* exit if logout fails */
+			gok_main_close ();
 		}
- 		gnome_client_request_save (client, GNOME_SAVE_GLOBAL, TRUE,
-			GNOME_INTERACT_ANY, FALSE, TRUE); /* code borrowed from 
-			gnome-control-center/ capplets/ accessibility/ at-properties/ 
-			main.c */
 	}
 	else if (strcmp((gchar*)data,"quit") == 0) {
 		gok_main_close();
@@ -3192,7 +3246,6 @@ void check_accessibility_cb ( GObject* o, gpointer* data )
 	else if (strcmp((gchar*)data,"continue") == 0) {
 		/* maybe change gok somehow to show user weakened status */
 	}
-
 }
 
 /**
