@@ -29,7 +29,7 @@
 #include <string.h>
 #include <stdlib.h>
 #include <ctype.h>
-#include <libgnome/gnome-macros.h>
+
 #include "word-complete.h"
 #include "gok-log.h"
 
@@ -85,20 +85,19 @@ static gchar *dictionary_full_path;
 
 /* 
  * This macro initializes GokTrieWordComplete with the GType system 
- *   and defines ..._class_init and ..._instance_init functions 
+ *   and defines ..._class_init and ..._init functions and a ..._parent_class pointer
  */
-GNOME_CLASS_BOILERPLATE (GokTrieWordComplete, gok_trie_wordcomplete,
-			 GokWordComplete, GOK_TYPE_WORDCOMPLETE)
+G_DEFINE_TYPE (GokTrieWordComplete, gok_trie_wordcomplete, GOK_TYPE_WORDCOMPLETE)
 
 static void
-gok_trie_wordcomplete_instance_init (GokTrieWordComplete *complete)
+gok_trie_wordcomplete_init (GokTrieWordComplete *complete)
 {
 }
 
 static void
 gok_trie_wordcomplete_class_init (GokTrieWordCompleteClass *klass)
 {
-	GokWordCompleteClass *word_complete_class = (GokWordCompleteClass *) klass;
+	GokWordCompleteClass *word_complete_class = GOK_WORDCOMPLETE_CLASS (klass);
 	word_complete_class->open = WordCompleteOpen;
 	word_complete_class->close = WordCompleteClose;
 	word_complete_class->increment_word_frequency = WordCompleteIncrementFrequency;
